<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Rules Tiered Discount by Amasty
 */

namespace Amasty\RulesTieredDiscount\Model\ResourceModel\SalesRule;

use Amasty\RulesTieredDiscount\Api\Data\TieredDiscountInterface;
use Amasty\RulesTieredDiscount\Model\ResourceModel\TieredDiscount\CollectionFactory;
use Amasty\RulesTieredDiscount\Model\Rule\Action\Discount\TieredDiscountPercent;
use Magento\Framework\Data\Collection;
use Magento\Framework\EntityManager\MetadataPool;
use Magento\Framework\EntityManager\Operation\ExtensionInterface;
use Magento\SalesRule\Api\Data\RuleInterface as SalesRuleInterface;
use Magento\SalesRule\Model\Rule;

class ReadHandler implements ExtensionInterface
{
    /**
     * @var MetadataPool
     */
    private $metadataPool;

    /**
     * @var CollectionFactory
     */
    private $collectionFactory;

    public function __construct(
        MetadataPool $metadataPool,
        CollectionFactory $collectionFactory
    ) {
        $this->metadataPool = $metadataPool;
        $this->collectionFactory = $collectionFactory;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     *
     * @param Rule $entity
     * @param array $arguments
     * @return Rule
     */
    public function execute($entity, $arguments = [])
    {
        if ($entity->getSimpleAction() !== TieredDiscountPercent::ACTION_NAME) {
            return $entity;
        }

        $linkField = $this->metadataPool->getMetadata(SalesRuleInterface::class)->getLinkField();
        $ruleLinkId = $entity->getDataByKey($linkField);
        $discountsCollection = $this->collectionFactory->create();
        $discountsCollection->addFieldToFilter('rule_id', $ruleLinkId)
            ->addOrder(TieredDiscountInterface::QUANTITY, Collection::SORT_ORDER_ASC);

        if ($ruleLinkId) {
            $attributes = $entity->getExtensionAttributes() ?: [];
            if (is_array($attributes)) {
                $attributes['amrules_tiered_discount'] = $discountsCollection->getItems();
            } else {
                $attributes->setAmrulesTieredDiscount($discountsCollection->getItems());
            }
            $entity->setExtensionAttributes($attributes);
        }

        return $entity;
    }
}
