<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Rules Tiered Discount by Amasty
 */

namespace Amasty\RulesTieredDiscount\Model\ResourceModel\SalesRule;

use Amasty\RulesTieredDiscount\Api\Data\TieredDiscountInterface;
use Amasty\RulesTieredDiscount\Model\ResourceModel\TieredDiscount as ResourceTieredDiscount;
use Amasty\RulesTieredDiscount\Model\Rule\Action\Discount\TieredDiscountPercent;
use Magento\Framework\EntityManager\MetadataPool;
use Magento\Framework\EntityManager\Operation\ExtensionInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\SalesRule\Api\Data\RuleInterface as SalesRuleInterface;
use Magento\SalesRule\Model\Rule;

class SaveHandler implements ExtensionInterface
{
    /**
     * @var MetadataPool
     */
    private $metadataPool;

    /**
     * @var ResourceTieredDiscount
     */
    private $resourceTieredDiscount;

    public function __construct(
        MetadataPool $metadataPool,
        ResourceTieredDiscount $resourceTieredDiscount
    ) {
        $this->metadataPool = $metadataPool;
        $this->resourceTieredDiscount = $resourceTieredDiscount;
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     *
     * @param Rule $entity
     * @param array $arguments
     * @return Rule
     * @throws LocalizedException
     */
    public function execute($entity, $arguments = [])
    {
        if ($entity->getSimpleAction() !== TieredDiscountPercent::ACTION_NAME) {
            return $entity;
        }

        $linkField = $this->metadataPool->getMetadata(SalesRuleInterface::class)->getLinkField();
        $attributes = $entity->getExtensionAttributes() ?: [];

        if (!isset($attributes['amrules_tiered_discount'])) {
            throw new LocalizedException(__('Tiered Discount is empty.'));
        }

        $ruleLinkId = (int)$entity->getDataByKey($linkField);
        $this->validate($attributes['amrules_tiered_discount']);

        $this->resourceTieredDiscount->saveDiscounts($ruleLinkId, $attributes['amrules_tiered_discount']);

        return $entity;
    }

    /**
     * @param array $tieredDiscounts [['quantity' => {qty}, 'discount' => {discount}], ...]
     * @return bool
     * @throws LocalizedException
     */
    private function validate(array $tieredDiscounts): bool
    {
        $quantity = [];
        $discount = [];

        foreach ($tieredDiscounts as $tieredDiscount) {
            if (($tieredDiscount[TieredDiscountInterface::QUANTITY] <= 0)
                || ($tieredDiscount[TieredDiscountInterface::DISCOUNT] <= 0)

            ) {
                throw new LocalizedException(__('Tiered Discount is incorrect. Only positive numbers are allowed.'));
            }
            $quantity[] = round((float)$tieredDiscount[TieredDiscountInterface::QUANTITY]);
            $discount[] = $tieredDiscount[TieredDiscountInterface::DISCOUNT];
        }

        if ((count($quantity) !== count(array_unique($quantity, SORT_NUMERIC)))
            || (count($discount) !== count(array_unique($discount, SORT_NUMERIC)))
        ) {
            throw new LocalizedException(__('We found duplicate values in the Tiered Discount.'));
        }

        return true;
    }
}
