<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Rules Tiered Discount by Amasty
 */

namespace Amasty\RulesTieredDiscount\Model\ResourceModel;

use Amasty\RulesTieredDiscount\Api\Data\TieredDiscountInterface;
use Magento\Framework\Model\ResourceModel\Db\AbstractDb;

class TieredDiscount extends AbstractDb
{
    public const MAIN_TABLE = 'amasty_amrules_tiered_discount';

    protected function _construct()
    {
        $this->_init(self::MAIN_TABLE, TieredDiscountInterface::ID);
    }

    /**
     * @param int $ruleId
     * @param array $discounts [['quantity' => {qty}, 'discount' => {discount}], ...]
     * @return void
     */
    public function saveDiscounts(int $ruleId, array $discounts): void
    {
        $connection = $this->getConnection();
        $tableName = $this->getTable(self::MAIN_TABLE);

        $connection->delete($tableName, ['rule_id = ?' => $ruleId]);

        $data = [];
        foreach ($discounts as $discountData) {
            $data[] = [
                'rule_id' => $ruleId,
                'quantity' => $discountData['quantity'],
                'discount' => $discountData['discount']
            ];
        }

        $connection->insertMultiple($tableName, $data);
    }

    /**
     * @param int $ruleId
     * @return array [['quantity' => {qty}, 'discount' => {discount}], ...]
     */
    public function getDataByRuleId(int $ruleId): array
    {
        $connection = $this->getConnection();
        $select = $connection->select()->from(
            $this->getTable(self::MAIN_TABLE),
            ['quantity', 'discount']
        )->where('rule_id = ?', $ruleId);

        return $connection->fetchAll($select);
    }
}
