<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Rules Tiered Discount by Amasty
 */

namespace Amasty\RulesTieredDiscount\Plugin\SalesRule\Model\Rule\DataProvider;

use Amasty\RulesTieredDiscount\Model\Rule\Action\Discount\TieredDiscountPercent;
use Magento\SalesRule\Model\Rule\DataProvider;

class PrepareDiscountData
{
    /**
     * @param DataProvider $subject
     * @param array|null $result
     * @return array|null
     */
    public function afterGetData(DataProvider $subject, ?array $result): ?array
    {
        if (!$result) {
            return $result;
        }

        foreach ($result as &$ruleData) {
            $tieredDiscountsData = [];
            if ($ruleData['simple_action'] == TieredDiscountPercent::ACTION_NAME
                && isset($ruleData['extension_attributes']['amrules_tiered_discount'])
            ) {
                foreach ($ruleData['extension_attributes']['amrules_tiered_discount'] as $tieredDiscountsEntity) {
                    $tieredDiscountsData[] = [
                        'quantity' => $tieredDiscountsEntity->getQuantity(),
                        'discount' => $tieredDiscountsEntity->getDiscount()
                    ];
                }

                $ruleData['extension_attributes']['amrules_tiered_discount'] = $tieredDiscountsData;
            }
        }

        return $result;
    }
}
