<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Rules Tiered Discount by Amasty
 */

namespace Amasty\RulesTieredDiscount\Setup\Patch\Schema;

use Amasty\RulesTieredDiscount\Api\Data\TieredDiscountInterface;
use Amasty\RulesTieredDiscount\Model\ResourceModel\TieredDiscount;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\EntityManager\MetadataPool;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\SchemaPatchInterface;
use Magento\SalesRule\Api\Data\RuleInterface;

class CreateSalesruleFk implements SchemaPatchInterface
{
    /**
     * @var ModuleDataSetupInterface
     */
    private $moduleDataSetup;
    
    /**
     * @var MetadataPool
     */
    private $metadata;

    public function __construct(
        MetadataPool $metadata,
        ModuleDataSetupInterface $moduleDataSetup
    ) {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->metadata = $metadata;
    }

    public function apply(): self
    {
        /** @var AdapterInterface $adapter */
        $adapter = $this->moduleDataSetup->getConnection();
        $tieredDiscountTableName = $this->moduleDataSetup->getTable(TieredDiscount::MAIN_TABLE);
        $salesruleTableName = $this->moduleDataSetup->getTable('salesrule');
        $linkField = $this->metadata->getMetadata(RuleInterface::class)->getLinkField();
        $adapter->addForeignKey(
            $adapter->getForeignKeyName(
                $tieredDiscountTableName,
                TieredDiscountInterface::RULE_ID,
                $salesruleTableName,
                $linkField
            ),
            $tieredDiscountTableName,
            TieredDiscountInterface::RULE_ID,
            $salesruleTableName,
            $linkField
        );

        return $this;
    }

    public function getAliases(): array
    {
        return [];
    }

    public static function getDependencies(): array
    {
        return [];
    }
}
