<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Rules Tiered Discount by Amasty
 */

namespace Amasty\RulesTieredDiscount\Test\Integration\Model\Discount;

use Amasty\RulesTieredDiscount\Model\Rule\Action\Discount\TieredDiscountPercent;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\ObjectManagerInterface;
use Magento\Quote\Api\Data\CartInterface;
use Magento\TestFramework\Helper\Bootstrap;
use Magento\TestFramework\Quote\Model\GetQuoteByReservedOrderId;
use PHPUnit\Framework\TestCase;

/**
 * @covers TieredDiscountPercent
 */
class TieredDiscountPercentTest extends TestCase
{
    private const ITEM_SKU = 'simple';

    /**
     * @var ObjectManagerInterface
     */
    protected $objectManager;

    /**
     * @var ProductInterface
     */
    private $product;

    /**
     * @var CartInterface
     */
    private $quote;

    /**
     * @magentoDataFixture Amasty_RulesTieredDiscount::Test/Integration/_files/cart_rule_tiered_discount_percent.php
     * @magentoDataFixture Amasty_Rules::Test/Integration/_files/products.php
     * @magentoDataFixture Amasty_RulesTieredDiscount::Test/Integration/_files/quote_with_address.php
     * @dataProvider dataProvider
     * @param int $qty
     * @param int $result
     */
    public function testCalculate(int $qty, int $result): void
    {
        $this->quote->addProduct($this->product, $qty);
        $this->quote->collectTotals();

        $this->assertEquals($result, $this->quote->getShippingAddress()->getBaseDiscountAmount());
    }

    public function dataProvider(): array
    {
        return [
            'small items qty' => [1, 0],
            'medium items qty' => [6, -5],
            'large items qty' => [15, -45]
        ];
    }

    protected function setUp(): void
    {
        $this->objectManager = Bootstrap::getObjectManager();
        $getQuoteByReservedOrderId = $this->objectManager->get(GetQuoteByReservedOrderId::class);
        $this->quote = $getQuoteByReservedOrderId->execute('test_amrules_tiered_discount');
        $productRepository = $this->objectManager->create(ProductRepositoryInterface::class);
        $this->product = $productRepository->get(self::ITEM_SKU, false, null, true);
    }
}
