<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Rules Tiered Discount by Amasty
 */

use Amasty\Rules\Model\Rule as AmRule;
use Amasty\RulesTieredDiscount\Model\Rule\Action\Discount\TieredDiscountPercent;
use Magento\AdvancedSalesRule\Model\Indexer\SalesRule\Processor;
use Magento\Customer\Model\GroupManagement;
use Magento\SalesRule\Model\ResourceModel\Rule as SalesRuleResource;
use Magento\SalesRule\Model\Rule;
use Magento\SalesRule\Model\RuleFactory;
use Magento\Store\Model\StoreManagerInterface;
use Magento\TestFramework\Helper\Bootstrap;

$objectManager = Bootstrap::getObjectManager();

$salesRuleResource = $objectManager->get(SalesRuleResource::class);
$salesRule = $objectManager->get(RuleFactory::class)->create();

$salesRule->setData(
    [
        'name' => 'Percent Discount: Buy X (Buy 1 for 10% off,  Buy 2 for 20% off, etc)',
        'is_active' => 1,
        'customer_group_ids' => [GroupManagement::NOT_LOGGED_IN_ID],
        'coupon_type' => Rule::COUPON_TYPE_NO_COUPON,
        'simple_action' => TieredDiscountPercent::ACTION_NAME,
        'stop_rules_processing' => 1,
        'website_ids' => [
            $objectManager->get(
                StoreManagerInterface::class
            )->getWebsite()->getId()
        ]
    ]
);

$amastyRuleModel = $objectManager->create(AmRule::class);
$amastyRuleModel->setApplyDiscountTo('asc');

$attributes = [
    'amrules' => $amastyRuleModel,
    'amrules_tiered_discount' => [
        ['quantity' => 3, 'discount' => 1],
        ['quantity' => 5, 'discount' => 10],
        ['quantity' => 8, 'discount' => 50]
    ]
];
$salesRule->setExtensionAttributes($attributes);

$salesRuleResource->save($salesRule);

if (class_exists(Processor::class)) {
    $indexer = $objectManager->get(Processor::class);
    $indexer->reindexRow($salesRule->getId());
}
